#include <ADC.h>

#define USE_TIMED_READS

int adc_0_pins[] = {14, 15, 16, 17};
int adc_1_pins[] = {14, 15};
uint8_t trigger_adc_0 = 0;
uint8_t trigger_adc_1 = 4;

uint8_t chain_len_0 = sizeof(adc_0_pins)/sizeof(adc_0_pins[0]);
uint8_t chain_len_1 = sizeof(adc_1_pins)/sizeof(adc_1_pins[0]);

ADC *adc = new ADC(); // adc object

#define PRREG(x) Serial.print(#x" 0x"); Serial.println(x,HEX)
volatile uint32_t val0, val1;
volatile uint32_t adcValues_0[10];
volatile uint32_t adcValues_1[10];

void setup() {
  while (!Serial && millis() < 5000) ;

  pinMode(LED_BUILTIN, OUTPUT);

  
  Serial.begin(9600);
  Serial.println("Setup ADC_0");

  adc->setAveraging(8); // set number of averages
  adc->setResolution(12); // set bits of resolution

  adc->adc1->setResolution(8);
  
  // always call the compare functions after changing the resolution!
  //adc->enableCompare(1.0/3.3*adc->getMaxValue(ADC_0), 0, ADC_0); // measurement will be ready if value < 1.0V
  //adc->enableCompareRange(1.0*adc->getMaxValue(ADC_1)/3.3, 2.0*adc->getMaxValue(ADC_1)/3.3, 0, 1, ADC_1); // ready if value lies out of [1.0,2.0] V

  setup_adc_hardware_trigger();
  
  Serial.println("End Setup");

}

void loop() {
  Serial.printf("ADC0:  %d  %d  %d  %d\n", adcValues_0[0], adcValues_0[1], adcValues_0[2], adcValues_0[3]);
  Serial.printf("ADC1:  %d  %d\n", adcValues_1[0], adcValues_1[1]);
  delay(500);
}

void setup_adc_hardware_trigger()
{
  adc_init();
  adc_etc_init();
  Serial.println("After ADC/ADC_ETC"); Serial.flush();

  //timer 1 = Q4_3, 2 - 4_0
  connectTimer(1, trigger_adc_0);
  connectTimer(2, trigger_adc_1);
  Serial.println("After XBAR"); Serial.flush();
  
  //USE_PIT0 (timer1)
  startTimer(1, 1000.0);   // 1 khz  1000 ADCs/sec
  //use QTIMER (timer2)
  startTimer(2, 3000.0);  // try at 20 hz for test...
}


void adc_init() {
  ADC1_CFG |= ADC_CFG_ADTRG;   // hardware trigger
  ADC1_HC0 = 16;   // ADC_ETC channel

  ADC2_CFG |= ADC_CFG_ADTRG;   // hardware trigger
  ADC2_HC0 = 16;   // ADC_ETC channel
  
}

void adc_etc_init() {

  adc_etc_reset();
 
  adc0_etc_ctrl_init(chain_len_0,         //chainLength_0, chainLength_1
                     adc_0_pins, //*adc_0_pinArray, *adc_1_pinArray
                     trigger_adc_0,          //trigger_adc_0, trigger_adc_1
                     false);     //bool DMA
  adc1_etc_ctrl_init(chain_len_1,         //chainLength_0, chainLength_1
                     adc_1_pins, //*adc_0_pinArray, *adc_1_pinArray
                     trigger_adc_1,          //trigger_adc_0, trigger_adc_1
                     false);     //bool DMA
  //  ADC_ETC_SetTriggerChainConfig(trigger#, channel Group#, pin1, pin2);
  ADC_ETC_SetTriggerChainConfig(ADC_0, trigger_adc_0, 0, adc_0_pins[0], adc_0_pins[1]);
  ADC_ETC_SetTriggerChainConfig(ADC_0, trigger_adc_0, 1, adc_0_pins[2], adc_0_pins[3]);
  ADC_ETC_SetTriggerChainConfig(ADC_1, trigger_adc_1, 0, adc_1_pins[0], adc_1_pins[1]);

  attachInterruptVector(IRQ_ADC_ETC0, adcetc0_isr);
  NVIC_ENABLE_IRQ(IRQ_ADC_ETC0);

}
